function [Icalc,R,varargout]=MCFit_sph(q,I,varargin)
%USAGE: [Icalc,R,varargout]=MCfit_sph(q,I,varargin)
%
%Starting out with a random distribution of sphere sizes within bounds,
%change the sphere with random size to have another size, and checking if
%it improves the fit or not. If it improves, keep, if not, discard.
%keep q small, perhaps only 100 values
%PV-pairs:
%'weighting', 'relative|unit|poisson', or A vector of size q or I, with
%the errors for each intensity value.
%'nsph', number of simulated spheres
%'nitr', number of iterations for the random move method
%'convcrit', convergence criterium, default 1, suitable for poisson
%weighting
%'graphics', (slow!) if set to 1 shows progress
%'exportgraphics', (slower!) saves all images so you can make a movie out
%of it
%'prior', prior guess of radii, for example from a previous iteration.
%'bounds', two-element vector with lower and upper radius bounds in the same size
%'qlim', a two-element vector with lower and upper limits for the q-range
%to fit. Note that this will affect the length of the output intensities
%accordingly.
%'param', a previously used parameters structured array (which is the
%'tolerance', a tolerance limit for the minimisation routine, default set
%to 10% of the convergence criterium to ensure a good minimisation. If the
%convergence criterium is set to zero, the tolerance must be set!
%optional third output argument.
%Optional fourth output argument is qfit vector.
%
%units as your measurement, e.g. [0 100] nm if your q is in nm^-1.
%
%If you use this program, please be so kind to put my name in the resulting
%publication
%This work by Brian Richard Pauw is licensed under a Creative Commons
%Attribution-ShareAlike 3.0 Unported License.

%%version 4.0, takes Ierr values (intensity error) to select or adjust
%weighting type (everything except gradient).
%version 5; deadwood cut. only random move still available. Cleaned version
%renamed to MCfit_sph.m

tic
display('initialising...')
%default settings
param.convcrit=1; %reasonable value for poisson weighting. any lower than this and we would be fitting noise
param.graphics=0;
param.exportgraphics=0; %saves the result in images
param.weighting='poisson';
param.method='randmove'; %the only method that seems to work so far
param.nsph=400;
param.nitr=[]; %to be postprocessed. Dependent on nsph
param.prior=[];
param.bounds=[pi./max(q)/2 pi/min(q)]; %some reasonable, but not necessarily correct, bounds. minimum/2 allows for "spillover"; spheres that do not actively contribute to the scattering pattern are placed here.
param.qlim=[min(q) max(q)];
param.Ierr=ones(size(q));
param.kansas=size(q); %shape to return output in.
param.tolx=[];%to be postprocessed. can be dependent on convcrit

%process input parameters
param=paramproc(param,varargin);

%postprocessing
if isempty(param.tolx);%not set in input
    if param.convcrit==0;
        error('if convergence criterium is set to 0, a value for the minimisation tolerance must be supplied')
    end
    param.tolx=0.1.*param.convcrit;
end
if isempty(param.nitr)
    param.nitr=10.*param.nsph; %each sphere moved 10 times.
end
if param.tolx>param.convcrit
    error('minimisation tolerance larger than convergence criterium. It is unlikely that convergence will be achieved')
end
%reshape inputs
q=reshape(q,[1 length(q)]);
I=reshape(I,[1 length(I)]);
param.Ierr=reshape(param.Ierr,[1 length(param.Ierr)]);
%apply weighting
switch param.weighting
    case 'err'
        %errors supplied, no need to do anything
    case 'relative'
        %relative weighting, Ierr=0.01*I
        param.Ierr=0.01.*I;
    case 'unit'
        param.Ierr=ones(size(q));
    case 'poisson'
        %error is the square root of the value
        Is=sort(I);
        mini=Is(find(Is>0,1,'first')); %minimum first positive, nonzero intensity value
        param.Ierr=sqrt(max(I,mini));
    otherwise
        error('weighting type not recognised, must be either "relative", "unit", "poisson", or a vector of length I with the error values')
end
%apply q limits
[q,I,param.Ierr]=qlimit(q,I,param);
%change the shape of the output to match the new limits
if length(q)~=max(param.kansas) %we have applied limits
    if param.kansas(1)>param.kansas(2)
        param.kansas=[length(q) 1];
    else
        param.kansas=[1 length(q)];
    end
end
%take or generate initial distribution
if ~isempty(param.prior)
    R=param.prior;
    R=reshape(R,[size(R,1)*size(R,2) 1]);
    if length(R)~=param.nsph
        warning('rgen:nsphdiscrepancy','number of spheres in prior %i, but number set in parameters is %i. \n using %i spheres.',length(R),param.nsph,length(R))
        param.nsph=length(R);
    end
else

    R=rand(param.nsph,1).*abs(diff(param.bounds))+min(param.bounds);
    R=reshape(R,[length(R) 1]);
end

%-----------------initialisation complete--------------------
display('calculating...')
%the real work is done here.
[R,Icalc,param]=mc_randmove(q,I,R,param);

%-----------------post-calculation work----------------------
%Afterthoughts. Roll credits, that sort of stuff.
Icalc=reshape(Icalc,param.kansas);
display(sprintf('converged to %s in %i steps and %i seconds, a speed of %s iterations per second',num2str(param.cs),param.stepi,round(toc),param.stepi/toc));
if nargout>=3
    varargout{1}=param;
end
if nargout>=4
    varargout{2}=q;
end
display('all done!');
end

function [R,Icalc,param]=mc_randmove(q,I,R,param)

%leftovers; Iprior is an intensity optimised in another step, which is
%added to the MC calculated intensity. Can be implemented at a later stage.
Iprior=zeros(size(I));


%calculate initial patterns
Fsph=sphereintensity_rayleigh(R*q);
V=(4/3*pi).*R.^3;
Iset=repmat(V,[1 length(q)]).^2.*abs(Fsph).^2;
Icalc=Iprior+sum(Iset,1)./sum(V.^2);
sc=1;
mini=min(I);

meritfunc=@(x) merit_csqr(I,Icalc,param,x);
[Fitparam,newFitval]=...
    fminsearchbnd(meritfunc,[sc mini],[0 -mini],[inf mini],optimset('MaxFunEvals',10000,'MaxIter',10000,'Tolx',param.tolx));

cs1=newFitval;
scc=Fitparam(1);
bgndc=Fitparam(2);

if param.graphics
    [handles,cslist]=initialise_graphics(q,I,Iprior+Icalc,sc,R,cs1,1e3,param.convcrit,param.exportgraphics);
end

Ist=Iset;
Vt=V;
stepi=1;
ri=0;
progressbar;
while stepi<param.nitr && cs1>param.convcrit
    ri=ri+1;
    if ri>param.nsph
        ri=1;
    end
    %ri=round(rand*(param.nsph-1))+1; %index of pattern to change ; nope,
    %sequentially will be just as fine.
    rc=rand.*abs(diff(param.bounds))+min(param.bounds);%factor to change it to)
    Vr=(4/3*pi).*rc.^3;

    Ist(ri,:)=Vr.^2.*sphereintensity_rayleigh(q.*rc).^2;

    Vt(ri)=Vr;
    Icalc=Iprior+sum(Ist,1)./sum(Vt.^2);

    meritfunc=@(x) sum(((I-(x(1).*Icalc+x(2)))./param.Ierr).^2)/(length(I));%merit_csqr(I,Icalc,param,x);
    [Fitparam,newFitval]=...
        fminsearchbnd(meritfunc,[scc bgndc],[0 -mini],[inf mini],optimset('MaxFunEvals',10000,'MaxIter',10000,'Tolx',param.tolx));

    cs=newFitval;
    sc=Fitparam(1);
    bgnd=Fitparam(2);



    %now test to see if it is any better.
    if cs<cs1 %new solution is better
        Iset(ri,:)=Ist(ri,:);
        R(ri)=rc;
        V(ri)=Vt(ri);
        cs1=cs;
        scc=sc;
        bgndc=bgnd;
    else
        Ist(ri,:)=Iset(ri,:); %revert the Ist values.
        Vt(ri)=V(ri);
    end %else just discard and continue.
    if param.graphics
        cslist(stepi)=cs1; %#ok<AGROW>
        if (stepi/500)==round(stepi/500) %only every 500 steps
            handles=update_graphics(handles,Iprior+Icalc.*scc+bgndc,sc,R,cslist,stepi);
        end
    end
    %show progressbar
    if (stepi/100)==round(stepi/100) %only every 100 steps
            progressbar(stepi/param.nitr);
    end
    stepi=stepi+1;
end
%destroy progressbar
progressbar(1)

Icalc=Iprior+sum(Iset,1)./sum(V.^2).*scc+bgndc; %I(0) should be one
param.sc=scc;
param.bgnd=bgndc;
param.cs=cs1;
param.stepi=stepi;
end

function cs=merit_csqr(I,Icalc,param,Fitparam)
cs=sum(((I-(Fitparam(1).*Icalc+Fitparam(2)))./param.Ierr).^2)/(length(I)); %weighted fitting
end


function handles=update_graphics(handles,Icalc,sc,R,cslist,stepi)
set(handles.newline,'YData',Icalc.*sc);
[histy,histx]=hist(R,50);
set(handles.hist,'xdata',histx,'ydata',histy);
axis(handles.histaxes,'tight')
set(handles.line,'XData',1:stepi,'YData',cslist(1:stepi),'ZData',ones(size(cslist(1:stepi))));
set(gca,'xlim',[1 stepi+100])
drawnow
%store
if param.exportgraphics
    print('-painters','-dpng',sprintf('-f%s',num2str(handles.fh)),sprintf('mcfit_%i.png',round(stepi/100)))
end
end

function [handles,cslist]=initialise_graphics(q,I,Icalc,sc,R,cs1,nsteps,convcrit,exportgraphics)
handles.fh=figure();
handles.dataaxes=subplot(2,2,[1 2]);
set(handles.dataaxes,'yscale','log');
handles.origline=line(q,I,'LineStyle','-','LineWidth',2,'Color',[0 0 0]);
handles.newline=line(q,Icalc.*sc,'LineStyle','-','LineWidth',2,'Color',[1 0 0]);
axis(handles.dataaxes,'tight')
xlabel('q, 1/\AA','interpreter','latex');ylabel('I (A.U.)','interpreter','latex');

handles.histaxes=subplot(2,2,3);
[histy,histx]=hist(R,50);
handles.hist=bar(histx,histy);
xlabel('Size, A','interpreter','latex'); %ylabel('Frequency (n)','interpreter','latex');
handles.convaxes=subplot(2,2,4);
handles.convline=semilogy(1:nsteps,repmat(convcrit,1,nsteps),'r-');
handles.line=line(1,cs1,'Color',[.8 0 0],'Marker','o','LineStyle','-','LineWidth',2,'Color',[0 0 0]);
xlabel('number of steps','interpreter','latex'); % ylabel('average mismatch (fraction)','interpreter','latex');
axis(handles.histaxes,'tight')

cslist=zeros(1,nsteps);
cslist(1)=cs1;
drawnow
%store
if exportgraphics
    print('-zbuffer','-dpng',sprintf('-f%s',num2str(handles.fh)),sprintf('mcfit_%i.png',round(stepi/100)))
end
end

function param=paramproc(param,vargi)

vi=0;
npv=length(vargi);
while vi<npv
    vi=vi+1;
    switch vargi{vi}
        case 'convcrit'
            vi=vi+1;
            param.convcrit=vargi{vi};
        case 'graphics'
            vi=vi+1;
            param.graphics=vargi{vi};
        case 'exportgraphics'
            vi=vi+1;
            param.exportgraphics=vargi{vi};
        case 'weighting'
            vi=vi+1;
            param.weighting=vargi{vi};
            if ~ischar(param.weighting) %a list of Ierr instead
                param.Ierr=param.weighting;
                param.weighting='err';
            end
            %         case 'method'
            %             vi=vi+1;
            %             param.method=vargi{vi};
        case 'nsph'
            vi=vi+1;
            param.nsph=vargi{vi};
            %         case 'nita'
            %             vi=vi+1;
            %             param.nita=vargi{vi};
        case 'nitr'
            vi=vi+1;
            param.nitr=vargi{vi};
            %         case 'nitg'
            %             vi=vi+1;
            %             param.nitg=vargi{vi};
            %         case 'nseq'
            %             vi=vi+1;
            %             param.nitr=vargi{vi};
        case 'prior'
            vi=vi+1;
            param.prior=vargi{vi};
        case 'bounds'
            vi=vi+1;
            param.bounds=vargi{vi};
        case 'qlim'
            vi=vi+1;
            param.qlim=vargi{vi};
            %         case 'maxmove'
            %             vi=vi+1;
            %             param.maxmove=vargi{vi};
            %         case 'forcemove'
            %             vi=vi+1;
            %             param.forcemove=vargi{vi};
        case 'param'
            vi=vi+1;
            param=vargi{vi};
            warning('inputprocessing:usagewarning','only parameter-value pairs after the param settings will be adjusted!')
        case 'tolerance'
            vi=vi+1;
            param.tolx=vargi{vi};

        otherwise
            vi=vi+1;
            warning('inputprocessing:invalidparameter','parameter %s is not one of the acceptable parameters. Check your usage',vargi{vi-1});
    end
end


end

function [qlim,Ilim,Ierr]=qlimit(q,I,param)
qlim=q(find(q>=min(param.qlim)&q<=max(param.qlim))); %#ok<FNDSB>
Ilim=I(find(q>=min(param.qlim)&q<=max(param.qlim)));     %#ok<FNDSB>
Ierr=param.Ierr(find(q>=min(param.qlim)&q<=max(param.qlim)));     %#ok<FNDSB>
end

function Fsph=sphereintensity_rayleigh(qR)

%this is the intensity calcultion at q for a sphere with radius R.
%as given in the paper by J.S. Pedersen, 1997, (eqn. 55)

Fsph=(qR>=0.05).*3.*(sin(qR)-qR.*cos(qR))./qR.^3+(qR<0.05).*(1-0.1.*qR.^2);
end
